/* 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 * Copyright 2008 Sun Microsystems, Inc. All rights reserved. Use is subject to license terms. 
 * 
 * This file is available and licensed under the following license:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 *   * Neither the name of Sun Microsystems nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package effectsplayground.control;

import javafx.animation.*;
import javafx.scene.*;
import javafx.scene.paint.*;
import javafx.scene.shape.*;
import javafx.scene.text.*;
import javafx.stage.*;

/**
 * @author campbell
 */

public def width = 100;
public def height = 20;

public class Button extends CustomNode {

    public var text = "Blank";
    public var action:function();

    var glowColor = Color.TRANSPARENT;
    def colorFade:Timeline = Timeline {
        keyFrames: [
            at (0s) {
                glowColor => Color.TRANSPARENT;
            },
            at (0.2s) {
                glowColor => Color.color(0.5, 0.5, 0.5, 0.5);
            }
        ]
    }

    override protected function create() : Node {
        var rect:Rectangle;
        var t:Text;
        Group {
            content: [
                rect = Rectangle {
                    cursor: Cursor.HAND
                    width: width
                    height: height
                    fill: LinearGradient {
                        startX: 0
                        startY: 0
                        endX: 0
                        endY: 1
                        stops: [
                            Stop { offset: 0.0 color: Color.rgb( 65,  65,  65) },
                            Stop { offset: 0.4 color: Color.rgb(  1,   1,   1) },
                            Stop { offset: 0.8 color: Color.rgb( 21,  21,  21) },
                            Stop { offset: 1.0 color: Color.rgb(  4,   4,   4) },
                        ]
                    }
                    stroke: LinearGradient {
                        startX: 0
                        startY: 0
                        endX: 0
                        endY: 1
                        stops: [
                            Stop { offset: 0.0 color: Color.rgb(  5,   5,   5) },
                            Stop { offset: 1.0 color: Color.rgb( 85,  85,  85) },
                        ]
                    }
                    onMouseEntered: function(e) {
                        colorFade.rate = 1;
                        colorFade.play();
                    }
                    onMouseExited: function(e) {
                        colorFade.rate = -1;
                        colorFade.play();
                    }
                    onMouseReleased: function(e) {
                        action();
                    }
                },
                Rectangle {
                    x: 1
                    y: 1
                    width: width - 1
                    height: height - 1
                    fill: bind glowColor
                },
                t = Text {
                    translateX: bind (rect.layoutBounds.width - t.layoutBounds.width) / 2
                    translateY: bind (rect.layoutBounds.height / 2) + 3
                    font: Font { size: 11 }
                    fill: Color.WHITE
                    content: bind text
                },
            ]
        }
    }
}

function run() {
Stage {
    width: 300
    height: 100
    scene: Scene {
        fill: Color.RED
        content: [
            Button {translateX: 20 translateY: 20}
        ]
    }
}
}
