/* 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 * Copyright 2008 Sun Microsystems, Inc. All rights reserved. Use is subject to license terms. 
 * 
 * This file is available and licensed under the following license:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 *   * Neither the name of Sun Microsystems nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package effectsplayground.control;

import javafx.animation.*;
import javafx.scene.*;
import javafx.scene.paint.*;
import javafx.scene.shape.*;

/**
 * @author campbell
 */

public class CloseButton extends CustomNode {
    def size = 8;
    var color = Color.rgb(153, 153, 153);

    def fade = Timeline {
        keyFrames: [
            at (0s) {
                color => Color.rgb(153, 153, 153);
            },
            at (0.2s) {
                color => Color.WHITE;
            }
        ]
    }

    override protected function create() : Node {
        Group {
            var g:Group;
            cursor: Cursor.HAND
            cache: true
            content: [
                Rectangle {
                    x: bind g.layoutBounds.minX
                    y: bind g.layoutBounds.minY
                    width: bind g.layoutBounds.width
                    height: bind g.layoutBounds.height
                    fill: Color.TRANSPARENT
                },
                g = Group {
                    content: [
                        Line {
                            startX: 0
                            startY: 0
                            endX: size
                            endY: size
                            stroke: bind color
                            strokeWidth: 3
                            strokeLineCap: StrokeLineCap.ROUND
                        },
                        Line {
                            startX: 0
                            startY: size
                            endX: size
                            endY: 0
                            stroke: bind color
                            strokeWidth: 3
                            strokeLineCap: StrokeLineCap.ROUND
                        },
                    ]
                }
            ]
            onMouseEntered: function(e) {
                fade.rate = 1;
                fade.play();
            }
            onMouseExited: function(e) {
                fade.rate = -1;
                fade.play();
            }
        }
    }
}
